import numpy as np
import scipy.integrate as sin
import os
import logging


from .transient_base_exporter import BaseExporter
from .transient_figures import (
    RawDataFigure,
    VoltageFigure,
    TempFigure,
    ExtrapolationFigure,
    ZCurveFigure,
    DerivFigure,
    FFTFigure,
    TimeSpecFigure,
    SumTimeSpecFigure,
    BackwardsImpFigure,
    BackwardsImpDerivFigure,
    CumulStrucFigure,
    DiffStrucFigure,
    LocalResistFigure,
    LocalGradientFigure,
    TheoCStrucFigure,
    TheoDiffStrucFigure,
    TheoTimeConstFigure,
    TheoSumTimeConstFigure,
    TheoImpDerivFigure,
    TheoImpFigure,
    TheoBackwardsImpFigure,
    OptimizeStrucFigure,
    TimeConstComparisonFigure,
    StrucComparisonFigure,
    TotalResistComparisonFigure,
    PredictionFigure,
    PredictionImpulseUsedFigure,
    ResidualFigure,
    BootZCurveFigure,
    BootDerivFigure,
    BootTimeSpecFigure,
    BootSumTimeSpecFigure,
    BootCumulStrucFigure,
)

logger = logging.getLogger("PyRthLogger")


class FigureExporter(BaseExporter):

    type = "figure"

    figure_registry = {
        "raw": ("look_at_raw_data", RawDataFigure),
        "voltage": ("look_at_voltage", VoltageFigure),
        "temp": ("look_at_temp", TempFigure),
        "extrpl": ("look_at_extrpl", ExtrapolationFigure),
        "impedance": ("look_at_impedance", ZCurveFigure),
        "deriv": ("look_at_deriv", DerivFigure),
        "fft": ("look_at_fft", FFTFigure),
        "time_spec": ("look_at_time_spec", TimeSpecFigure),
        "sum_time_spec": ("look_at_sum_time_spec", SumTimeSpecFigure),
        "back_imp": ("look_at_backwards_impedance", BackwardsImpFigure),
        "back_deriv": ("look_at_backwards_imp_deriv", BackwardsImpDerivFigure),
        "cumul_struc": ("look_at_cumul_struc", CumulStrucFigure),
        "diff_struc": ("look_at_diff_struc", DiffStrucFigure),
        "local_resist": ("look_at_local_resist", LocalResistFigure),
        "local_gradient": ("look_at_local_gradient", LocalGradientFigure),
        "theo_cstruc": ("look_at_theo_cstruc", TheoCStrucFigure),
        "theo_diff_struc": ("look_at_theo_diff_struc", TheoDiffStrucFigure),
        "theo_time_const": ("look_at_theo_time_const", TheoTimeConstFigure),
        "theo_sum_time_const": (
            "look_at_theo_sum_time_const",
            TheoSumTimeConstFigure,
        ),
        "theo_imp_deriv": ("look_at_theo_imp_deriv", TheoImpDerivFigure),
        "theo_impedance": ("look_at_theo_impedance", TheoImpFigure),
        "theo_back_imp": (
            "look_at_theo_backwards_impedance",
            TheoBackwardsImpFigure,
        ),
        "optimize_struc": ("look_at_optimize_struc", OptimizeStrucFigure),
        "time_const_comparison": (
            "look_at_time_const_comparison",
            TimeConstComparisonFigure,
        ),
        "struc_comparison": ("look_at_struc_comparison", StrucComparisonFigure),
        "total_resist_comparison": (
            "look_at_total_resist_comparison",
            TotalResistComparisonFigure,
        ),
        "prediction": ("look_at_prediction", PredictionFigure),
        "prediction_imp": (
            "look_at_prediction_figure",
            PredictionImpulseUsedFigure,
        ),
        "residual": ("look_at_residual", ResidualFigure),
        "boot_impedance": ("look_at_boot_impedance", BootZCurveFigure),
        "boot_deriv": ("look_at_boot_deriv", BootDerivFigure),
        "boot_time_spec": ("look_at_boot_time_spec", BootTimeSpecFigure),
        "boot_sum_time_spec": ("look_at_boot_sum_time_spec", BootSumTimeSpecFigure),
        "boot_cumul_struc": ("look_at_boot_cumul_struc", BootCumulStrucFigure),
    }

    def __init__(self, figures):
        super().__init__()
        self.figures = figures

    def save_all_figures(self) -> None:
        """Save all figures generated by modules to PNG files."""

        logger.info("Saving figures")
        logger.debug(f"Number of figures to save: {len(self.figures)}")

        # Save each figure using its module's parameters
        for fig_name, fig_obj in self.figures.items():
            try:
                output_dir = os.path.join(fig_obj.output_dir, fig_obj.label)
                os.makedirs(output_dir, exist_ok=True)
                filename = os.path.join(output_dir, f"{fig_name}_.png")
                fig_obj.fig.savefig(filename)
                fig_obj.close()
                logger.debug(f"Figure '{fig_name}' saved successfully.")

            except Exception as e:
                logger.error(f"Error saving figure '{fig_name}': {str(e)}")
                continue

    def initialize_registered_figures(self, keys, module):
        """Instantiate and store figure objects for keys whose condition attribute evaluates True."""
        for key in keys:
            if key in self.figure_registry:
                cond_attr, figure_class = self.figure_registry[key]
                condition = getattr(module, cond_attr, False)
                if condition:
                    logger.debug(f"Creating figure for key '{key}'")
                    self.figures[key + "_" + module.label] = figure_class(
                        module
                    )  # Pass the module to the figure class
                else:
                    logger.debug(f"Condition for key '{key}' not met.")
            else:
                logger.error(f"Key '{key}' not found in figure registry")

    def extrapol_data_handler(self, module):
        logger.debug("extrapol_data_handler called")
        self.initialize_registered_figures(["extrpl"], module)

    def voltage_data_handler(self, module):
        logger.debug("voltage_data_handler called")
        self.initialize_registered_figures(["voltage"], module)

    def temp_data_handler(self, module):
        logger.debug("temp_data_handler called")
        self.initialize_registered_figures(["raw", "temp"], module)

    def impedance_data_handler(self, module):
        logger.debug("impedance_data_handler called")
        self.initialize_registered_figures(["impedance", "deriv"], module)

    def fft_data_handler(self, module):
        logger.debug("fft_data_handler called")
        self.initialize_registered_figures(["fft"], module)

    def time_spec_data_handler(self, module):
        logger.debug("time_spec_data_handler called")

        self.initialize_registered_figures(
            ["time_spec", "sum_time_spec", "back_imp", "back_deriv"], module
        )

    def structure_function_data_handler(self, module):
        logger.debug("structure_function_data_handler called")

        self.initialize_registered_figures(
            ["cumul_struc", "diff_struc", "local_resist", "local_gradient"], module
        )

    def theo_structure_function_data_handler(self, module):
        logger.debug("theo_structure_function_data_handler called")

        self.initialize_registered_figures(["theo_cstruc", "theo_diff_struc"], module)

    def theo_data_handler(self, module):
        logger.debug("theo_data_handler called")

        self.initialize_registered_figures(
            [
                "theo_time_const",
                "theo_sum_time_const",
                "theo_imp_deriv",
                "theo_impedance",
            ],
            module,
        )

    def theo_compare_data_handler(self, module):
        logger.debug("theo_compare_data_handler called")

        self.initialize_registered_figures(["theo_back_imp"], module)

    def optimize_data_handler(self, module):
        logger.debug("optimize_data_handler called")
        self.initialize_registered_figures(["optimize_struc"], module)

    def comparison_data_handler(self, module):
        logger.debug("comparison_data_handler called")
        self.initialize_registered_figures(
            ["time_const_comparison", "struc_comparison", "total_resist_comparison"],
            module,
        )

    def prediction_data_handler(self, module):
        logger.debug("prediction_data_handler called")

        self.initialize_registered_figures(["prediction", "prediction_imp"], module)

    def residual_data_handler(self, module):
        logger.debug("residual_data_handler called")
        self.initialize_registered_figures(["residual"], module)

    def boot_data_handler(self, module):
        logger.debug("boot_data_handler called")

        self.initialize_registered_figures(
            [
                "boot_impedance",
                "boot_deriv",
                "boot_time_spec",
                "boot_sum_time_spec",
                "boot_cumul_struc",
            ],
            module,
        )
